//----------------------------------------------------------------------
//
// GTK - The GIMP Toolkit
// Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
//
// Modified by Göran Hass, Raditex AB, gorhas@raditex.nu
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
//--------------------------------------------------------------------------------


static volatile char idtxt[] = "version 0.1";

#include <math.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include <gtk/gtkmain.h>
#include <gtk/gtksignal.h>
#include <gtk/gtkstyle.h>

#include "gtkhhbar.h"

#define SCROLL_DELAY_LENGTH  300
#define HHBAR_DEFAULT_SIZE 30

/* Forward declarations */
static void gtk_hhbar_class_init               (GtkHHbarClass    *klass);
static void gtk_hhbar_init                     (GtkHHbar         *hhbar);
static void gtk_hhbar_destroy                  (GtkObject        *object);
static void gtk_hhbar_realize                  (GtkWidget        *widget);
static void gtk_hhbar_size_request             (GtkWidget      *widget,
					       GtkRequisition *requisition);
static void gtk_hhbar_size_allocate            (GtkWidget     *widget,
					       GtkAllocation *allocation);
static gint gtk_hhbar_expose                   (GtkWidget        *widget,
					       GdkEventExpose   *event);
static gint gtk_hhbar_button_press             (GtkWidget        *widget,
					       GdkEventButton   *event);
static gint gtk_hhbar_button_release           (GtkWidget        *widget,
					       GdkEventButton   *event);
static gint gtk_hhbar_motion_notify            (GtkWidget        *widget,
					       GdkEventMotion   *event);
static gint gtk_hhbar_timer                    (GtkHHbar         *hhbar);

static void gtk_hhbar_update_mouse             (GtkHHbar *hhbar, gint x, gint y, gint move_point);
static void gtk_hhbar_update                   (GtkHHbar *hhbar);
static void gtk_hhbar_adjustment_changed       (GtkAdjustment    *adjustment,
					       gpointer          data);
static void gtk_hhbar_adjustment_value_changed (GtkAdjustment    *adjustment,
					       gpointer          data);

static double delta1, delta2;


static int ix;

/* Local data */
static GtkWidgetClass *parent_class = NULL;

static int move_point = 0;

guint
gtk_hhbar_get_type ()
{
  static guint hhbar_type = 0;

  if (!hhbar_type)
    {
      GtkTypeInfo hhbar_info =
      {
	"GtkHHbar",
	sizeof (GtkHHbar),
	sizeof (GtkHHbarClass),
	(GtkClassInitFunc) gtk_hhbar_class_init,
	(GtkObjectInitFunc) gtk_hhbar_init,
	/* (GtkArgSetFunc) NULL, */
	/* (GtkArgGetFunc) NULL, */
	(GtkClassInitFunc) NULL,
      };

      hhbar_type = gtk_type_unique (gtk_widget_get_type (), &hhbar_info);
    }

  return hhbar_type;
}

static void
gtk_hhbar_class_init (GtkHHbarClass *class)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;

  object_class = (GtkObjectClass*) class;
  widget_class = (GtkWidgetClass*) class;


  parent_class = gtk_type_class (gtk_widget_get_type ());

  object_class->destroy = gtk_hhbar_destroy;

  widget_class->realize = gtk_hhbar_realize;
  widget_class->expose_event = gtk_hhbar_expose;
  widget_class->size_request = gtk_hhbar_size_request;
  widget_class->size_allocate = gtk_hhbar_size_allocate;
  widget_class->button_press_event = gtk_hhbar_button_press;
  widget_class->button_release_event = gtk_hhbar_button_release;
  widget_class->motion_notify_event = gtk_hhbar_motion_notify;
}

static void
gtk_hhbar_init (GtkHHbar *hhbar)
{
  hhbar->button = 0;
  hhbar->policy = GTK_UPDATE_CONTINUOUS;
  hhbar->timer = 0;

  hhbar->width = 100;
  hhbar->height = 30;
  hhbar->handle_width = 0;
  hhbar->value = 0.0;

  hhbar->old_value = 0.0;
  hhbar->old_lower = 0.0;
  hhbar->old_upper = 0.0;
  hhbar->adjustment = NULL;
}

GtkWidget*
gtk_hhbar_new(GtkAdjustment *adjustment)
{
  GtkHHbar *hhbar;

  hhbar = gtk_type_new(gtk_hhbar_get_type());

  if (!adjustment)
    adjustment = (GtkAdjustment*)gtk_adjustment_new(0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

  gtk_hhbar_set_adjustment(hhbar, adjustment);

  return GTK_WIDGET (hhbar);
}

static void
gtk_hhbar_destroy(GtkObject *object)
{
  GtkHHbar *hhbar;

  g_return_if_fail (object != NULL);
  g_return_if_fail (GTK_IS_HHBAR (object));

  hhbar = GTK_HHBAR (object);

  if (hhbar->adjustment)
  {
    gtk_object_unref(GTK_OBJECT (hhbar->adjustment));
    hhbar->adjustment = NULL;
  }

  if (GTK_OBJECT_CLASS (parent_class)->destroy)
    (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}

GtkAdjustment*
gtk_hhbar_get_adjustment (GtkHHbar *hhbar)
{
  g_return_val_if_fail (hhbar != NULL, NULL);
  g_return_val_if_fail (GTK_IS_HHBAR (hhbar), NULL);

  return hhbar->adjustment;
}

void
gtk_hhbar_set_update_policy(GtkHHbar *hhbar, GtkUpdateType  policy)
{
  g_return_if_fail (hhbar != NULL);
  g_return_if_fail (GTK_IS_HHBAR (hhbar));

  hhbar->policy = policy;
}

void
gtk_hhbar_set_adjustment(GtkHHbar *hhbar, GtkAdjustment *adjustment)
{
  g_return_if_fail (hhbar != NULL);
  g_return_if_fail (GTK_IS_HHBAR (hhbar));

  if (hhbar->adjustment)
    {
      gtk_signal_disconnect_by_data (GTK_OBJECT (hhbar->adjustment), (gpointer) hhbar);
      gtk_object_unref (GTK_OBJECT (hhbar->adjustment));
    }

  hhbar->adjustment = adjustment;

  gtk_object_ref (GTK_OBJECT (hhbar->adjustment));

  gtk_signal_connect (GTK_OBJECT (adjustment), "changed",
		      (GtkSignalFunc) gtk_hhbar_adjustment_changed,
		      (gpointer) hhbar);

  gtk_signal_connect (GTK_OBJECT (adjustment), "value_changed",
		      (GtkSignalFunc) gtk_hhbar_adjustment_value_changed,
		      (gpointer) hhbar);

  hhbar->old_value = adjustment->value;
  hhbar->old_lower = adjustment->lower;
  hhbar->old_upper = adjustment->upper;

  gtk_hhbar_update (hhbar);
}

static void
gtk_hhbar_realize (GtkWidget *widget)
{
  GdkWindowAttr attributes;
  gint attributes_mask;

 GdkColor white     = { 0, 0xffff, 0xffff, 0xffff };
 GdkColor red       = { 0, 0xffff, 0x0000, 0x0000 };
 GdkColor green     = { 0, 0x8000, 0xffff, 0x0000 };
 GdkColor yellow    = { 0, 0xffff, 0xffff, 0x0000 };
 GdkColor blue      = { 0, 0x0000, 0x8000, 0xffff };
 GdkColor gray      = { 0, 0xa000, 0xa000, 0xa000 }; 
 GdkColor lightgray = { 0, 0xc000, 0xc000, 0xc000 }; 

  GtkStyle *defstyle;
  GtkStyle *style;

  //gdk_color_parse( "gray80", &lightgray);

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_HHBAR (widget));

  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);

  attributes.x = widget->allocation.x;
  attributes.y = widget->allocation.y;
  attributes.width = widget->allocation.width;
  attributes.height = widget->allocation.height;

  attributes.wclass = GDK_INPUT_OUTPUT;
  attributes.window_type = GDK_WINDOW_CHILD;
  attributes.event_mask = gtk_widget_get_events (widget) | 
    GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK | 
    GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK |
    GDK_POINTER_MOTION_HINT_MASK;
  attributes.visual = gtk_widget_get_visual (widget);
  attributes.colormap = gtk_widget_get_colormap (widget);

  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;

  widget->window = gdk_window_new (widget->parent->window, &attributes, attributes_mask);

  gdk_color_alloc ( gdk_colormap_get_system(), &white );
  gdk_color_alloc ( gdk_colormap_get_system(), &red );
  gdk_color_alloc ( gdk_colormap_get_system(), &green );
  gdk_color_alloc ( gdk_colormap_get_system(), &yellow );
  gdk_color_alloc ( gdk_colormap_get_system(), &blue );
  gdk_color_alloc ( gdk_colormap_get_system(), &gray );
  gdk_color_alloc ( gdk_colormap_get_system(), &lightgray );

  defstyle = gtk_widget_get_default_style();

  style = gtk_style_copy( defstyle );
  style->fg[GTK_STATE_NORMAL] = white;
  style->bg[GTK_STATE_NORMAL] = gray;
  style->fg[0] = white;
  style->bg[0] = white;
  style->bg[0] = gray;
  style->bg[1] = white;
  style->bg[2] = blue;
  style->bg[3] = lightgray;
  style->bg[4] = green;
  style->bg[5] = yellow;

  gtk_widget_set_style ( widget, style );  
  widget->style = gtk_style_attach (widget->style, widget->window);
  
  gdk_window_set_user_data(widget->window, widget);
  gtk_style_set_background(widget->style, widget->window, GTK_STATE_NORMAL);
}

static void 
gtk_hhbar_size_request(GtkWidget *widget, GtkRequisition *requisition)
{
  requisition->width  = HHBAR_DEFAULT_SIZE;
  requisition->height = HHBAR_DEFAULT_SIZE;
}

static void
gtk_hhbar_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
{
  GtkHHbar *hhbar;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_HHBAR (widget));
  g_return_if_fail (allocation != NULL);

  widget->allocation = *allocation;
  hhbar = GTK_HHBAR (widget);

  if (GTK_WIDGET_REALIZED (widget))
    {
      gdk_window_move_resize (widget->window,
			      allocation->x, allocation->y,
			      allocation->width, allocation->height);
    }

  hhbar->width  = allocation->width;
  hhbar->height = allocation->height;
}

static gint
gtk_hhbar_expose(GtkWidget *widget, GdkEventExpose *event)
{
  GtkHHbar *hhbar;
  GdkPoint points[12];

  gdouble hhbar_unit, handle_lower_pos, handle_upper_pos;

  gint yc,w,h;

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_HHBAR (widget), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);

  if (event->count > 0)
    return FALSE;
                      
  hhbar = GTK_HHBAR (widget);
              
  gdk_window_clear_area(widget->window, 0, 0,
			widget->allocation.width,
			widget->allocation.height);
  
  gtk_draw_shadow(widget->style,widget->window,GTK_STATE_ACTIVE,GTK_SHADOW_IN, 0,0,
			widget->allocation.width,
			widget->allocation.height);

  //if (GTK_WIDGET_SENSITIVE(widget) == FALSE) return FALSE;

  yc = widget->allocation.height/2 ;

  /* ???? */

  hhbar_unit = ((gdouble)widget->allocation.width /
                (gdouble)(hhbar->adjustment->upper-hhbar->adjustment->lower));

  handle_lower_pos = (
      ((gdouble)hhbar->adjustment->value -
       (gdouble) hhbar->adjustment->page_size / (gdouble)2.0) -
       (gdouble)hhbar->adjustment->lower ) * hhbar_unit;

  handle_upper_pos = (
      ((gdouble)hhbar->adjustment->value +
       (gdouble)hhbar->adjustment->page_size / (gdouble)2.0) -
       (gdouble)hhbar->adjustment->lower ) * hhbar_unit ;

  /* Draw pillar */

  points[0].x = handle_lower_pos;
  points[0].y = yc - (hhbar->height/2 - 3);

  points[1].x = handle_lower_pos;
  points[1].y = yc + (hhbar->height/2 - 2);

  points[2].x = handle_upper_pos;
  points[2].y = yc + (hhbar->height/2 - 2);

  points[3].x = handle_upper_pos;
  points[3].y = yc - (hhbar->height/2 - 3); 

  points[4].x = points[0].x;
  points[4].y = points[0].y;

  gdk_draw_polygon(widget->window, widget->style->bg_gc[ix], TRUE, points, 5);

  w = points[2].x - points[0].x - 21;

  /* Draw the left handle */

  points[0].x = handle_lower_pos - 0;
  points[0].y = yc - (hhbar->height/2 - 3);

  points[1].x = handle_lower_pos - 0;
  points[1].y = yc + (hhbar->height/2 - 2);

  points[2].x = handle_lower_pos + 10;
  points[2].y = yc + (hhbar->height/2 - 2);

  points[3].x = handle_lower_pos + 10;
  points[3].y = yc - (hhbar->height/2 - 3); 

  points[4].x = points[0].x;
  points[4].y = points[0].y;

  gtk_draw_shadow(widget->style,widget->window,GTK_STATE_ACTIVE,GTK_SHADOW_OUT,
		  points[0].x,points[0].y,points[2].x-points[0].x,points[1].y-points[0].y);

  h  = points[1].y-points[0].y;

  // Draw the center shadow
  gtk_draw_shadow(widget->style,widget->window,GTK_STATE_ACTIVE,GTK_SHADOW_OUT,
		  points[2].x,points[0].y,w,h);

  // Draw the right handle

  points[0].x = handle_upper_pos - 10;
  points[0].y = yc - (hhbar->height/2 - 3);

  points[1].x = handle_upper_pos - 10;
  points[1].y = yc + (hhbar->height/2 - 2);

  points[2].x = handle_upper_pos - 0;
  points[2].y = yc + (hhbar->height/2 - 2);

  points[3].x = handle_upper_pos - 0;
  points[3].y = yc - (hhbar->height/2 - 3); 

  points[4].x = points[0].x;
  points[4].y = points[0].y;

  gtk_draw_shadow(widget->style,widget->window,GTK_STATE_ACTIVE,GTK_SHADOW_OUT,
		  points[0].x,points[0].y,points[2].x-points[0].x,points[1].y-points[0].y);
  
  //widget->style->fg_gc[GTK_STATE_NORMAL],
	
  return FALSE;  
}

static gint
gtk_hhbar_button_press(GtkWidget *widget, GdkEventButton *event)
{
  GtkHHbar *hhbar;

  double hhbar_handle_left_pos, hhbar_handle_right_pos;
  double hhbar_unit;

  g_return_val_if_fail(widget != NULL, FALSE);
  g_return_val_if_fail(GTK_IS_HHBAR(widget), FALSE);
  g_return_val_if_fail(event != NULL, FALSE);

  hhbar = GTK_HHBAR (widget);

  // Use only left mouse button: 
  if (event->button != 1) return FALSE;

  /* Determine if button press was within pointer region - we 
     do this by computing the parallel and perpendicular distance of
     the point where the mouse was pressed from the line passing through
     the pointer */


  hhbar_unit = (widget->allocation.width/(hhbar->adjustment->upper - hhbar->adjustment->lower));
  
  hhbar_handle_left_pos = 
      ((hhbar->adjustment->value - hhbar->adjustment->page_size/2) - hhbar->adjustment->lower)
      * hhbar_unit;         
  
  hhbar_handle_right_pos =
      ((hhbar->adjustment->value + hhbar->adjustment->page_size/2) - hhbar->adjustment->lower)
      * hhbar_unit;         
  
  delta1 = event->x - hhbar_handle_left_pos;
  delta2 = hhbar_handle_right_pos - event->x;

  hhbar->x_delta =
      hhbar_handle_left_pos + (hhbar_handle_right_pos - hhbar_handle_left_pos)/2 - event->x;
  
  move_point = 0;

  if ((delta1 >= 0) && (delta1 <= 10))
  {
    move_point = 1;
  } 
  else if ((delta2 >= 0) && (delta2 <= 10))
  {
    move_point = 2;
  }
  else if ((hhbar_handle_left_pos <= event->x) &&
           (event->x <= hhbar_handle_right_pos))
  {
    move_point = 3;
  }
  else if (delta1 < 0)
  {
    hhbar->adjustment->page_size += hhbar->adjustment->page_increment;
    gtk_adjustment_set_value(hhbar->adjustment,
        hhbar->adjustment->value - hhbar->adjustment->page_increment/2.0);
  }
  else if (delta2 < 0)
  {
    hhbar->adjustment->page_size += hhbar->adjustment->page_increment;
    gtk_adjustment_set_value(hhbar->adjustment,
        hhbar->adjustment->value + hhbar->adjustment->page_increment/2.0);
  }

  if (move_point)
  {
    gtk_grab_add (widget);
    hhbar->button = event->button;
    gtk_hhbar_update_mouse(hhbar, event->x, event->y, move_point);
  }

  return FALSE;
}

static gint
gtk_hhbar_button_release(GtkWidget *widget, GdkEventButton *event)
{
  GtkHHbar *hhbar;

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_HHBAR (widget), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);

  hhbar = GTK_HHBAR(widget);

  if (hhbar->button == event->button)
    {
      gtk_grab_remove(widget);

      hhbar->button = 0;
      move_point = 0;

      if (hhbar->policy == GTK_UPDATE_DELAYED)
	gtk_timeout_remove(hhbar->timer);
                          
      if ((hhbar->policy != GTK_UPDATE_CONTINUOUS) &&
	  (hhbar->old_value != hhbar->adjustment->value))
	gtk_signal_emit_by_name (GTK_OBJECT (hhbar->adjustment), "value_changed");
    }

  return FALSE;
}

static gint
gtk_hhbar_motion_notify(GtkWidget *widget, GdkEventMotion *event)
{
  GtkHHbar *hhbar;
  GdkModifierType mods;
  gint x, y, mask;

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_HHBAR (widget), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);

  hhbar = GTK_HHBAR (widget);

  if (hhbar->button != 0)
  {
      x = event->x;
      y = event->y;

      if (event->is_hint || (event->window != widget->window))
	  gdk_window_get_pointer (widget->window, &x, &y, &mods);
       else
          mods = 0xFFFFFFFF;
      
      switch (hhbar->button)
      {
      case 1:
	  mask = GDK_BUTTON1_MASK;
	  break;
      case 2:
	  mask = GDK_BUTTON2_MASK;
	  break;
      case 3:
	  mask = GDK_BUTTON3_MASK;
	  break;
      default:
	  mask = 0;
	  break;
      }
      if (mods & mask)
	  gtk_hhbar_update_mouse(hhbar, x, y, move_point);
  }
  
  return FALSE;
}

static gint
gtk_hhbar_timer(GtkHHbar *hhbar)
{
  g_return_val_if_fail (hhbar != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_HHBAR (hhbar), FALSE);

  if (hhbar->policy == GTK_UPDATE_DELAYED)
    gtk_signal_emit_by_name(GTK_OBJECT(hhbar->adjustment), "value_changed");

  return FALSE;
}

static void
gtk_hhbar_update_mouse(GtkHHbar *hhbar, gint x, gint y, gint move_point)
{
  double hhbar_unit, hhbar_handle_left_pos, hhbar_handle_right_pos;
  g_return_if_fail (hhbar != NULL);
  g_return_if_fail (GTK_IS_HHBAR (hhbar));

  hhbar_unit = (hhbar->width/(hhbar->adjustment->upper - hhbar->adjustment->lower));
 
  hhbar_handle_left_pos = (
      (hhbar->adjustment->value-hhbar->adjustment->page_size/2)-hhbar->adjustment->lower
      ) * hhbar_unit;
  
  hhbar_handle_right_pos = (
      (hhbar->adjustment->value + hhbar->adjustment->page_size/2) - hhbar->adjustment->lower
      ) * hhbar_unit;         

  switch(move_point) {
  case 1:
  {
      hhbar_handle_left_pos = x - delta1;

      if (hhbar_handle_left_pos < 0.0)
      {
	  hhbar_handle_left_pos = 0.0;
      }

      if ( hhbar_handle_left_pos > hhbar_handle_right_pos - 25)
	   hhbar_handle_left_pos = hhbar_handle_right_pos - 25;

      hhbar->adjustment->page_size = 
	  (hhbar_handle_right_pos-hhbar_handle_left_pos)/hhbar_unit;

      hhbar->adjustment->value = hhbar_handle_right_pos/hhbar_unit +
	  hhbar->adjustment->lower -
	  hhbar->adjustment->page_size/2;
      

      break;
  }
  case 2:
  {
      hhbar_handle_right_pos = x + delta2;

      if (hhbar->adjustment->lower + (hhbar_handle_right_pos - 0)/hhbar_unit >
	  hhbar->adjustment->upper)
      {
	  hhbar_handle_right_pos =
      (hhbar->adjustment->upper - hhbar->adjustment->lower - 0)* hhbar_unit;	  
      }
      
      if ( hhbar_handle_right_pos < hhbar_handle_left_pos + 25)
	   hhbar_handle_right_pos = hhbar_handle_left_pos + 25;

      hhbar->adjustment->page_size = 
	  (hhbar_handle_right_pos-hhbar_handle_left_pos)/hhbar_unit;

      hhbar->adjustment->value = hhbar_handle_right_pos/hhbar_unit +
	  hhbar->adjustment->lower -
	  hhbar->adjustment->page_size/2;

      break;
  } 
  case 3:
  {
      x += hhbar->x_delta;

      hhbar->adjustment->value = hhbar->adjustment->lower + x/hhbar_unit;

      if (hhbar->adjustment->value - hhbar->adjustment->page_size/2 < hhbar->adjustment->lower)
      {
	  hhbar->adjustment->value = hhbar->adjustment->lower + hhbar->adjustment->page_size/2;
      }

      if (hhbar->adjustment->value + hhbar->adjustment->page_size/2 > hhbar->adjustment->upper)
      {
	  hhbar->adjustment->value = hhbar->adjustment->upper - hhbar->adjustment->page_size/2;
      }
      break;
  }
  default:
      return;
  }
  
  if (hhbar->policy == GTK_UPDATE_CONTINUOUS)
  {
      gtk_signal_emit_by_name (GTK_OBJECT (hhbar->adjustment), "value_changed");
  }
  else
  {
      gtk_signal_emit_by_name (GTK_OBJECT (hhbar->adjustment), "value_changed");
      return;

      gtk_widget_draw(GTK_WIDGET(hhbar), NULL);
      if (hhbar->policy == GTK_UPDATE_DELAYED)
      {
	  if (hhbar->timer)
	      gtk_timeout_remove(hhbar->timer);
	  
	  hhbar->timer = gtk_timeout_add(SCROLL_DELAY_LENGTH,
					 (GtkFunction)gtk_hhbar_timer,
					 (gpointer)hhbar);
      }
  }
}

static void
gtk_hhbar_update(GtkHHbar *hhbar)
{
  g_return_if_fail (hhbar != NULL);
  g_return_if_fail (GTK_IS_HHBAR (hhbar));
                      
  if (hhbar->adjustment->value < hhbar->adjustment->lower)
      gtk_adjustment_set_value(hhbar->adjustment, hhbar->adjustment->lower);

  if (hhbar->adjustment->value > hhbar->adjustment->upper)
      gtk_adjustment_set_value(hhbar->adjustment, hhbar->adjustment->upper);

  hhbar->value = hhbar->adjustment->value;

  gtk_widget_draw (GTK_WIDGET(hhbar), NULL);
}

static void
gtk_hhbar_adjustment_changed(GtkAdjustment *adjustment, gpointer data)
{
  GtkHHbar *hhbar;

  g_return_if_fail(adjustment != NULL);
  g_return_if_fail(data != NULL);

  hhbar = GTK_HHBAR (data);

  gtk_hhbar_update(hhbar);
  
  hhbar->old_value = adjustment->value;
  hhbar->old_lower = adjustment->lower;
  hhbar->old_upper = adjustment->upper;
}

static void
gtk_hhbar_adjustment_value_changed(GtkAdjustment *adjustment,
				   gpointer       data)
{
  GtkHHbar *hhbar;

  g_return_if_fail (adjustment != NULL);
  g_return_if_fail (data != NULL);

  hhbar = GTK_HHBAR (data);

  if (hhbar->old_value != adjustment->value)
    {
      gtk_hhbar_update(hhbar);
      hhbar->old_value = adjustment->value;
    }
}

void gtk_hhbar_on(GtkHHbar *hhbar, gboolean state, gboolean gate)
{
  GdkEventExpose event;

  g_return_if_fail (hhbar != NULL);
  g_return_if_fail (GTK_IS_HHBAR (hhbar));


  if (state)
  {
    if (gate) ix = 2;
    else      ix = 6;
  }
  else
  {
    ix = 3;
  }

      event.type = GDK_EXPOSE;
      event.send_event = TRUE;
      event.window = hhbar->widget.window;
      event.count = 0;
      
      event.area.x = 0;
      event.area.y = 0;
      event.area.width  = hhbar->width;
      event.area.height = hhbar->height;
      
      gdk_window_ref (event.window);
      gtk_widget_event (GTK_WIDGET (hhbar), (GdkEvent *)&event);
      gdk_window_unref (event.window);
  
}
 
//----------------------------------------------------------------------
//  EOF
//----------------------------------------------------------------------
